@@ -4,6 +4,7 @@
 import typing
 
 from .TBlock import TBlock
+from .TList import TList
 
 
 
@@ -18,8 +19,8 @@ class TSection(object):
 	#
 	# Constructor method.
 	#
-	# @param		str title				The title of this section
-	# @param		(str|TBlock|TSection)[] contentBlocks		The paragraphs of this section
+	# @param		str title										The title of this section
+	# @param		(str|TBlock|TList|TSection)[] contentBlocks		The paragraphs of this section
 	#
 	def __init__(self, title:str, contentBlocks:typing.Sequence):
 		assert isinstance(title, str)
@@ -42,7 +43,7 @@ class TSection(object):
 	#
 
 	@property
-	def contentBlocks(self) -> list:
+	def contentBlocks(self) -> list:				# -> typing.List[typing.Union[str,TBlock,TList,TSection]]
 		return list(self.__contentBlocks)
 	#
 
@@ -61,6 +62,8 @@ class TSection(object):
 		b = None
 		if isinstance(content, TBlock):
 			b = content
+		elif isinstance(content, TList):
+			b = content
 		elif isinstance(content, TSection):
 			b = content
 		elif isinstance(content, str):
